using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.IO;
using System.Windows.Forms;

namespace HCNavegador
{
    class Cliente
    {
        public IPEndPoint Resolver(Uri endereco)
        {
            try
            {
                int porta = 1989;
                switch (endereco.Scheme.ToLower())
                {
                    case "tftp":
                        porta = 68;
                        break;
                    case "hccom":
                        porta = 1989;
                        break;
                    default:
                        throw new Exception("Tipo de servidor invlido. Aceitos: tftp, hccom");
                        break;
                }
                if (endereco.Port > 0) porta = endereco.Port;
                IPHostEntry ip = Dns.GetHostEntry(endereco.Host);
                if (ip.AddressList.Length == 0)
                {
                    return null;
                }
                IPEndPoint ponto = new IPEndPoint(ip.AddressList[0], porta);
                return ponto;
            }
            catch (Exception)
            {
                return null;
            }
        }

        public Retorno LerTexto(Uri endereco)
        {
            Retorno ret = new Retorno();
            try
            {
                List<byte> dadosRet = new List<byte>();
                IPEndPoint serv = Resolver(endereco);
                ushort posicao = 1;
                Socket conex = new Socket(serv.AddressFamily, SocketType.Dgram, ProtocolType.Udp);
                Pacote.GerarRequisicaoDeLeitura(endereco.PathAndQuery, Modos.Texto).Enviar(conex, serv);
                conex.ReceiveTimeout = 1000;
                Pacote recebido = new Pacote();
                while (recebido.Receber(conex, serv))
                {
                    if (recebido.Comando == Comandos.Erro)
                    {
                        ret.Conteudo = dadosRet.ToArray();
                        ret.Posicao = recebido.LerNumero(2);
                        ret.Sucesso = false;
                        ret.Mensagem = Encoding.ASCII.GetString(recebido.Dados, 2, recebido.Dados.Length - 3).TrimEnd('\0');
                        return ret;
                    }
                    if (recebido.LerNumero(2) == posicao)
                    {
                        byte[] tmp = new byte[recebido.Dados.Length - 2];
                        Array.Copy(recebido.Dados, 2, tmp, 0, tmp.Length);
                        dadosRet.AddRange(tmp);
                        Pacote.GerarProcessado(posicao).Enviar(conex, serv);
                        posicao++;
                    }
                    if (recebido.Dados.Length < 514) break;
                }
                conex.Close();
                ret.Sucesso = true;
                ret.Conteudo = dadosRet.ToArray();
                return ret;
            }
            catch (Exception e)
            {
                ret.Sucesso = false;
                ret.Mensagem = e.Message;
                return ret;
            }
        }

        public Retorno SalvarTexto(Uri endereco, FileStream saida)
        {
            Retorno ret = new Retorno();
            try
            {
                IPEndPoint serv = Resolver(endereco);
                ushort posicao = 1;
                Socket conex = new Socket(serv.AddressFamily, SocketType.Dgram, ProtocolType.Udp);
                Pacote.GerarRequisicaoDeLeitura(endereco.PathAndQuery, Modos.Texto).Enviar(conex, serv);
                conex.ReceiveTimeout = 1000;
                Pacote recebido = new Pacote();
                while (recebido.Receber(conex, serv))
                {
                    if (recebido.Comando == Comandos.Erro)
                    {
                        ret.Posicao = recebido.LerNumero(2);
                        ret.Sucesso = false;
                        ret.Mensagem = Encoding.ASCII.GetString(recebido.Dados, 2, recebido.Dados.Length - 3).TrimEnd('\0');
                        return ret;
                    }
                    if (recebido.LerNumero(2) == posicao)
                    {
                        saida.Write(recebido.Dados, 2, recebido.Dados.Length - 2);
                        Pacote.GerarProcessado(posicao).Enviar(conex, serv);
                        posicao++;
                    }
                    if (recebido.Dados.Length < 514) break;
                }
                saida.Flush();
                conex.Close();
                ret.Sucesso = true;
                return ret;
            }
            catch (Exception e)
            {
                ret.Sucesso = false;
                ret.Mensagem = e.Message;
                return ret;
            }
        }

        public Retorno Ler(Uri endereco)
        {
            Retorno ret = new Retorno();
            try
            {
                List<byte> dadosRet = new List<byte>();
                IPEndPoint serv = Resolver(endereco);
                ushort posicao = 1;
                Socket conex = new Socket(serv.AddressFamily, SocketType.Dgram, ProtocolType.Udp);
                Pacote.GerarRequisicaoDeLeitura(endereco.PathAndQuery, Modos.Binario).Enviar(conex, serv);
                conex.ReceiveTimeout = 1000;
                Pacote recebido = new Pacote();
                while (recebido.Receber(conex, serv))
                {
                    Application.DoEvents();
                    if (recebido.Comando == Comandos.Erro)
                    {
                        ret.Conteudo = dadosRet.ToArray();
                        ret.Posicao = recebido.LerNumero(0);
                        ret.Sucesso = false;
                        ret.Mensagem = Encoding.UTF8.GetString(recebido.Dados, 2, recebido.Dados.Length - 3).TrimEnd('\0');
                        return ret;
                    }
                    if (recebido.LerNumero(0) == posicao)
                    {
                        byte[] tmp = new byte[recebido.Dados.Length - 2];
                        Array.Copy(recebido.Dados, 2, tmp, 0, tmp.Length);
                        dadosRet.AddRange(tmp);
                        Pacote.GerarProcessado(posicao).Enviar(conex, serv);
                        posicao++;
                        ret.Sucesso = true;
                    }
                    if (recebido.Dados.Length < 514) break;
                }
                conex.Close();
                ret.Conteudo = dadosRet.ToArray();
                return ret;
            }
            catch (Exception e)
            {
                ret.Sucesso = false;
                ret.Mensagem = e.Message;
                return ret;
            }
        }

        public Retorno Salvar(Uri endereco, FileStream saida)
        {
            Retorno ret = new Retorno();
            try
            {
                IPEndPoint serv = Resolver(endereco);
                ushort posicao = 1;
                Socket conex = new Socket(serv.AddressFamily, SocketType.Dgram, ProtocolType.Udp);
                Pacote.GerarRequisicaoDeLeitura(endereco.PathAndQuery, Modos.Binario).Enviar(conex, serv);
                conex.ReceiveTimeout = 1000;
                Pacote recebido = new Pacote();
                while (recebido.Receber(conex, serv))
                {
                    if (recebido.Comando == Comandos.Erro)
                    {
                        ret.Posicao = recebido.LerNumero(2);
                        ret.Sucesso = false;
                        ret.Mensagem = Encoding.ASCII.GetString(recebido.Dados, 2, recebido.Dados.Length - 3).TrimEnd('\0');
                        return ret;
                    }
                    if (recebido.LerNumero(2) == posicao)
                    {
                        saida.Write(recebido.Dados, 2, recebido.Dados.Length - 2);
                        Pacote.GerarProcessado(posicao).Enviar(conex, serv);
                        posicao++;
                    }
                    if (recebido.Dados.Length < 514) break;
                }
                saida.Flush();
                conex.Close();
                ret.Sucesso = true;
                return ret;
            }
            catch (Exception e)
            {
                ret.Sucesso = false;
                ret.Mensagem = e.Message;
                return ret;
            }
        }
    }

    class Pacote
    {
        private byte[] _dados;

        public byte[] Dados
        {
            get { return _dados; }
            set { _dados = value; }
        }

        private Comandos _comando;

        public Comandos Comando
        {
            get { return _comando; }
            set { _comando = value; }
        }

        public ushort LerNumero(int pos)
        {
            return (ushort)((Dados[pos] << 8) & 0xff00 | Dados[pos + 1]);
        }

        private static string converterModo(Modos modo)
        {
            switch (modo)
            {
                case Modos.Desconhecido:
                    return "";
                case Modos.Texto:
                    return "netascii";
                case Modos.Binario:
                    return "octet";
                case Modos.Email:
                    return "mail";
                default:
                    return "";
            }
        }

        public static Pacote GerarRequisicaoDeLeitura(string arquivo, Modos modo)
        {
            Pacote ret = new Pacote();
            ret.Comando = Comandos.RequisicaoDeLeitura;
            List<byte> conteudo = new List<byte>();
            conteudo.AddRange(Encoding.UTF8.GetBytes(arquivo));
            conteudo.Add(0);
            conteudo.AddRange(Encoding.UTF8.GetBytes(converterModo(modo)));
            conteudo.Add(0);
            ret.Dados = conteudo.ToArray();
            return ret;
        }

        public static Pacote GerarRequisicaoDeGravacao(string arquivo, Modos modo)
        {
            Pacote ret = new Pacote();
            ret.Comando = Comandos.RequisicaoDeGravacao;
            List<byte> conteudo = new List<byte>();
            conteudo.AddRange(Encoding.UTF8.GetBytes(arquivo));
            conteudo.Add(0);
            conteudo.AddRange(Encoding.UTF8.GetBytes(converterModo(modo)));
            conteudo.Add(0);
            ret.Dados = conteudo.ToArray();
            return ret;
        }

        public static Pacote GerarDados(ushort posicao, byte[] dados)
        {
            Pacote ret = new Pacote();
            ret.Comando = Comandos.Dados;
            List<byte> conteudo = new List<byte>();
            conteudo.Add((byte)((posicao >> 8) & 0xff));
            conteudo.Add((byte)(posicao & 0xff));
            conteudo.AddRange(dados);
            ret.Dados = conteudo.ToArray();
            return ret;
        }

        public static Pacote GerarProcessado(ushort posicao)
        {
            Pacote ret = new Pacote();
            ret.Comando = Comandos.Processado;
            List<byte> conteudo = new List<byte>();
            conteudo.Add((byte)((posicao >> 8) & 0xff));
            conteudo.Add((byte)(posicao & 0xff));
            ret.Dados = conteudo.ToArray();
            return ret;
        }

        public void Enviar(Socket conexao, IPEndPoint servidor)
        {
            List<byte> pacote = new List<byte>();
            pacote.Add(0);
            pacote.Add((byte)Comando);
            pacote.AddRange(Dados);
            conexao.SendTo(pacote.ToArray(), pacote.Count, SocketFlags.None, servidor);
        }

        public bool Receber(Socket conexao, IPEndPoint servidor)
        {
            try
            {
                EndPoint ep = (EndPoint)servidor;
                byte[] tmp = new byte[1024];
                int tam = conexao.ReceiveFrom(tmp, ref ep);
                Comando = (Comandos)tmp[1];
                Dados = new byte[tam - 2];
                Array.Copy(tmp, 2, Dados, 0, tam - 2);
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }

    class Retorno
    {
        private bool _sucesso = false;

        public bool Sucesso
        {
            get { return _sucesso; }
            set { _sucesso = value; }
        }

        private int _posicao = -1;

        public int Posicao
        {
            get { return _posicao; }
            set { _posicao = value; }
        }

        private string _mensagem = "";

        public string Mensagem
        {
            get { return _mensagem; }
            set { _mensagem = value; }
        }

        private byte[] _conteudo = new byte[] { };

        public byte[] Conteudo
        {
            get { return _conteudo; }
            set { _conteudo = value; }
        }
        public string ConteudoTexto
        {
            get { return Encoding.UTF8.GetString(_conteudo); }
            set { _conteudo = Encoding.UTF8.GetBytes(value); }
        }

    }

    enum Comandos
    {
        Desconhecido = 0,
        RequisicaoDeLeitura = 1,
        RequisicaoDeGravacao = 2,
        Dados = 3,
        Processado = 4,
        Erro = 5
    }

    enum Modos
    {
        Desconhecido = 0,
        Texto = 1,
        Binario = 2,
        Email = 3
    }
}
