using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace HCNavegador
{
    public partial class NavegadorControle : UserControl
    {
        private PrincForm _tela;

        public PrincForm Tela
        {
            get { return _tela; }
            set { _tela = value; }
        }


        public NavegadorControle(PrincForm tela, string endereco)
        {
            Tela = tela;
            InitializeComponent();
            enderecoCampo.Text = endereco;
        }

        public TabPage Aba
        {
            get
            {
                return (TabPage)Parent;
            }
        }

        enum ModoProcessamento
        {
            Texto,
            Link,
            Titulo
        }

        private void processar(string codigo)
        {
            ModoProcessamento modo = ModoProcessamento.Texto;
            string[] linhas = codigo.Replace("\r", "").Split('\n');
            conteudoCampo.Clear();
            Font padrao = enderecoCampo.Font;
            Font titulo = new Font(enderecoCampo.Font, FontStyle.Bold);
            foreach (string linha in linhas)
            {
                string filtrado = linha.Trim('\t', ' ');
                string exibir = filtrado + " ";
                bool enter = false;
                if (filtrado.StartsWith("#"))
                {
                    conteudoCampo.SelectionFont = titulo;
                    exibir = "";
                    foreach (char c in filtrado)
                    {
                        if(!(exibir == "" & c == '#'))
                            exibir += c;
                    }
                    exibir = exibir.Trim(' ', '\t');
                    enter = true;
                }
                else if (filtrado.StartsWith("@"))
                {
                    if (filtrado.StartsWith("@@"))
                    {
                        exibir = filtrado.Substring(1) + " ";
                    }
                    else
                    {
                        exibir = "";
                        string[] conteudoFiltrado = filtrado.Substring(1).Split('=');
                        string campo = conteudoFiltrado[0].ToLower().Trim(' ', '\t');
                        string valor = "";
                        if (conteudoFiltrado.Length > 1)
                        {
                            valor = conteudoFiltrado[1].Trim(' ', '\t');
                        }
                        switch (campo)
                        {
                            case "title":
                                Aba.Text = valor;
                                break;
                            default:
                                break;
                        }
                    }
                }
                else if (filtrado == "")
                    enter = true;
                else
                {
                    exibir = "";
                    modo = ModoProcessamento.Texto;
                    foreach (char c in filtrado)
                    {
                        switch (modo)
                        {
                            case ModoProcessamento.Texto:
                                switch (c)
                                {
                                    case '[':
                                        modo = ModoProcessamento.Link;
                                        break;
                                    default:
                                        exibir += c;
                                        break;
                                }
                                break;
                            case ModoProcessamento.Link:
                                
                                break;
                            case ModoProcessamento.Titulo:
                                break;
                            default:
                                break;
                        }
                    }
                }
                conteudoCampo.AppendText(exibir);
                conteudoCampo.SelectionFont = padrao;
                if (enter)
                    conteudoCampo.AppendText(Environment.NewLine);
            }
            
        }

        private void fecharAbaBotao_Click(object sender, EventArgs e)
        {
            Tela.FecharAba(Aba);
        }

        private void irBotao_Click(object sender, EventArgs e)
        {
            try
            {
                Cliente cli = new Cliente();
                statusProgresso.Value = 25;
                Uri endereco = new Uri(enderecoCampo.Text);
                if (endereco.PathAndQuery.Split('?')[0] == "/")
                {
                    endereco = new Uri(endereco.ToString() + "index.rmd");
                }
                enderecoCampo.Text = endereco.ToString();
                if (endereco.Host == "")
                {
                    processar(@"@title=HCCom
# Protocolo HCCom

O protocolo HCCom  retrocompatvel com o TFTP, podendo acessar pginas em servidores TFTP.


Formato de endereo aceitos:

- tftp://servidor/arquivo

- hccom://servidor/arquivo");

                }
                else
                {
                    statusProgresso.Value = 50;
                    statusRotulo.Text = "Conectando";
                    Retorno ret = cli.Ler(endereco);
                    if (ret.Sucesso)
                    {
                        string arquivo = endereco.PathAndQuery.Split('?')[0];
                        statusProgresso.Value = 75;
                        if (Path.GetFileName(arquivo) == "dir")
                        {
                            string conteudoDir = "@title=Contedo" + Environment.NewLine +
                                "# Diretrio " + Path.GetDirectoryName(arquivo) + Environment.NewLine + Environment.NewLine;
                            for (int i = 0; i < ret.Conteudo.Length; i += 64)
                            {
                                if (ret.Conteudo[i] == 10)
                                {
                                    conteudoDir += "(DIR) ";
                                }
                                else if (ret.Conteudo[i] == 20)
                                {
                                    conteudoDir += "(ARQ) ";
                                }
                                else
                                {
                                    conteudoDir += "(???) ";
                                }
                                conteudoDir += Encoding.UTF8.GetString(ret.Conteudo, i + 3, 60).TrimEnd('\0');
                                conteudoDir += string.Format(" ({0} KiB)", (((ret.Conteudo[i + 1] << 8) & 0xff00 | ret.Conteudo[i + 2]) / 1024));
                                conteudoDir += Environment.NewLine + Environment.NewLine;
                            }

                            processar(conteudoDir);
                        }
                        else if (Path.GetExtension(arquivo).ToLower() == ".rmd")
                        {
                            processar(ret.ConteudoTexto);
                        }
                        else
                        {
                            if (MessageBox.Show(string.Format("Deseja salvar o arquivo '{0}'?", Path.GetFileName(arquivo)), "Baixar", MessageBoxButtons.YesNoCancel) == DialogResult.Yes)
                            {
                                SaveFileDialog sfd = new SaveFileDialog();
                                sfd.Title = "Salvar arquivo";
                                sfd.FileName = Path.GetFileName(arquivo);
                                sfd.Filter = "Arquivo " + Path.GetExtension(arquivo) +
                                    "|*" + Path.GetExtension(arquivo);
                                if (sfd.ShowDialog() == DialogResult.OK)
                                {
                                    File.WriteAllBytes(sfd.FileName, ret.Conteudo);
                                }
                            }
                        }
                    }
                    else
                    {
                        string msgErro = "@title=Erro" + Environment.NewLine +
                            "# Ocorreu um erro durante a solicitao" + Environment.NewLine + Environment.NewLine;
                        if (ret.Mensagem.Length > 0)
                        {
                            msgErro += ret.Mensagem;
                        }
                        else
                        {
                            msgErro += "No foi possvel se conectar ao servidor.";
                        }
                        processar(msgErro);
                    }
                }
                statusRotulo.Text = "Concludo";
            }
            catch (Exception ex)
            {
                statusRotulo.Text = "Erro: " + ex.Message;
            }
            statusProgresso.Value = 0;
        }

        private void conteudoCampo_LinkClicked(object sender, LinkClickedEventArgs e)
        {

        }

        private void NavegadorControle_Load(object sender, EventArgs e)
        {
            if (enderecoCampo.Text.Length == 0)
            {
                enderecoCampo.Text = "hccom://";
                return;
            }
            irBotao.PerformClick();
        }

        private void enderecoCampo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                e.SuppressKeyPress = true;
                irBotao.PerformClick();
            }
        }
    }
}
